@@ -24,6 +24,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ******************************************************************/
 
 #include "shutdowndlg.h"
+//include <ksmserver_debug.h>
 
 #include <QApplication>
 #include <QDesktopWidget>
@@ -44,15 +45,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <Plasma/PluginLoader>
 #include <kiconloader.h>
-#include <klocale.h>
+#include <klocalizedstring.h>
 #include <kuser.h>
 #include <Solid/PowerManagement>
 #include <kwindowsystem.h>
 #include <netwm.h>
 #include <kdeclarative/kdeclarative.h>
+#include <ksharedconfig.h>
 
 #include <stdio.h>
-#include <kxerrorhandler.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 
@@ -162,7 +163,7 @@ KSMShutdownDlg::KSMShutdownDlg( QWindow* parent,
         fileName = theme;
 
     if (QFile::exists(fileName)) {
-        //kDebug() << "Using QML theme" << fileName;
+        //qCDebug(KSMSERVER) << "Using QML theme" << fileName;
         setSource(QUrl::fromLocalFile(fileName));
     } else {
         qWarning() << "Couldn't find a theme for the Shutdown dialog" << fileName;
@@ -247,7 +248,6 @@ void KSMShutdownDlg::slotHalt()
     accept();
 }
 
-
 void KSMShutdownDlg::slotSuspend(int spdMethod)
 {
     m_bootOption.clear();
