@@ -19,6 +19,7 @@
 
 #include "SplashWindow.h"
 
+#include <QGuiApplication>
 #include <QQmlContext>
 #include <QQuickItem>
 #include <QKeyEvent>
@@ -26,7 +27,10 @@
 #include <QTimer>
 #include <QStandardPaths>
 #include <QSurfaceFormat>
-#include <shellpluginloader.h>
+#include <KSharedConfig>
+
+#include <Plasma/Package>
+#include <Plasma/PluginLoader>
 
 SplashWindow::SplashWindow(bool testing, bool window)
     : QQuickView(),
@@ -85,10 +89,19 @@ void SplashWindow::setGeometry(const QRect& rect)
     QQuickView::setGeometry(rect);
 
     if (oldGeometryEmpty) {
-        ShellPluginLoader::init();
-        Plasma::Package pkg = Plasma::PluginLoader::self()->loadPackage("Plasma/LookAndFeel");
-        pkg.setPath("org.kde.lookandfeel");
 
-        setSource(QUrl::fromLocalFile(pkg.filePath("splashmainscript")));
+        Plasma::Package package = Plasma::PluginLoader::self()->loadPackage("Plasma/LookAndFeel");
+        KConfigGroup cg(KSharedConfig::openConfig("kdeglobals"), "KDE");
+        const QString packageName = cg.readEntry("LookAndFeelPackage", QString());
+        if (!packageName.isEmpty()) {
+            package.setPath(packageName);
+        };
+
+        const QString theme = QGuiApplication::arguments().at(1);
+        if (!theme.startsWith("--")) {
+            package.setPath(theme);
+        }
+
+        setSource(QUrl::fromLocalFile(package.filePath("splashmainscript")));
     }
 }
