@@ -161,6 +161,10 @@ ShellCorona::ShellCorona(QObject *parent)
 
     new Osd(this);
     m_screenConfiguration = KScreen::Config::current();
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 4, 0))
+    connect(qGuiApp, &QGuiApplication::screenRemoved, this, &ShellCorona::screenRemoved);
+#endif
 }
 
 ShellCorona::~ShellCorona()
@@ -308,6 +312,10 @@ void ShellCorona::load()
 
 void ShellCorona::primaryOutputChanged()
 {
+    if (m_loading) {
+        QTimer::singleShot(500, this, SLOT(primaryOutputChanged()));
+    }
+
     if (m_views.isEmpty()) {
         return;
     }
@@ -319,8 +327,10 @@ void ShellCorona::primaryOutputChanged()
 
     QScreen *oldPrimary = m_views[0]->screen();
     QScreen *newPrimary = outputToScreen(output);
-    if (!newPrimary)
+    if (!newPrimary || newPrimary == oldPrimary) {
+        CHECK_SCREEN_INVARIANTS
         return;
+    }
     qDebug() << "primary changed!" << oldPrimary->name() << newPrimary->name();
 
     foreach (DesktopView *view, m_views) {
@@ -354,6 +364,7 @@ void ShellCorona::primaryOutputChanged()
 #ifndef NDEBUG
 void ShellCorona::screenInvariants() const
 {
+    Q_ASSERT(!m_loading);
     Q_ASSERT(m_views.count() <= QGuiApplication::screens().count());
     QScreen *s = m_views.isEmpty() ? nullptr : m_views[0]->screen();
     KScreen::Output *primaryOutput = findPrimaryOutput();
@@ -600,6 +611,16 @@ QList<PanelView *> ShellCorona::panelsForScreen(QScreen *screen) const
     return ret;
 }
 
+DesktopView* ShellCorona::desktopForScreen(QScreen* screen) const
+{
+    foreach (DesktopView *v, m_views) {
+        if (v->screen() == screen) {
+            return v;
+        }
+    }
+    return Q_NULLPTR;
+}
+
 void ShellCorona::removeView(int idx)
 {
     if (idx < 0 || idx >= m_views.count() || m_views.isEmpty()) {
@@ -647,6 +668,14 @@ void ShellCorona::removeView(int idx)
     }
 }
 
+void ShellCorona::screenRemoved(QScreen* screen)
+{
+    if (DesktopView* v = desktopForScreen(screen))
+        remove(v);
+//     addOutput(screenToOutput(screen));
+    m_reconsiderOutputsTimer.start();
+}
+
 void ShellCorona::outputEnabledChanged()
 {
     addOutput(qobject_cast<KScreen::Output *>(sender()));
@@ -683,24 +712,25 @@ void ShellCorona::reconsiderOutputs()
 
     foreach (KScreen::Output *out, m_screenConfiguration->connectedOutputs()) {
         if (!out->isEnabled()) {
+//             qDebug() << "skip screen" << out << desktopForScreen(outputToScreen(out));
             continue;
         }
 
         if (m_redundantOutputs.contains(out)) {
             if (!isOutputRedundant(out)) {
+//                 qDebug() << "not redundant anymore" << out;
                 addOutput(out);
             }
         } else if (isOutputRedundant(out)) {
-            QScreen *screen = outputToScreen(out);
-            for (int i = 0; i < m_views.count(); ++i) {
-                if (m_views[i]->screen() == screen) {
-                    removeView(i);
-                    break;
-                }
-            }
+//             qDebug() << "new redundant screen" << out;
+
+            if (DesktopView* v = desktopForScreen(outputToScreen(out)))
+                remove(v);
 
             m_redundantOutputs.insert(out);
         }
+//         else
+//             qDebug() << "fine screen" << out;
     }
 
     CHECK_SCREEN_INVARIANTS
@@ -745,7 +775,7 @@ void ShellCorona::addOutput(KScreen::Output *output)
 
     QScreen* newScreen = insertScreen(screen, insertPosition);
 
-    DesktopView *view = new DesktopView(this, screen);
+    DesktopView *view = new DesktopView(this, newScreen);
 
     Plasma::Containment *containment = createContainmentForActivity(m_activityController->currentActivity(), m_views.count());
     Q_ASSERT(containment);
@@ -1175,6 +1205,9 @@ Plasma::Containment *ShellCorona::setContainmentTypeForScreen(int screen, const
     view->rootObject()->setFocus(true, Qt::MouseFocusReason);
     QTimer::singleShot(2500, oldContainment, SLOT(destroy()));
 
+    emit availableScreenRectChanged();
+    emit availableScreenRegionChanged();
+
     return newContainment;
 }
 
@@ -1381,19 +1414,17 @@ int ShellCorona::screenForContainment(const Plasma::Containment *containment) co
     }
 
     //Failed? fallback on lastScreen()
-    int i = 0;
     //lastScreen() is the correct screen for panels
     //It is also correct for desktops *that have the correct activity()*
     //a containment with lastScreen() == 0 but another activity,
     //won't be associated to a screen
-    qDebug() << "ShellCorona screenForContainment: " << containment << " Last screen is " << containment->lastScreen();
-    for (KScreen::Output *output : sortOutputs(m_screenConfiguration->outputs())) {
+//     qDebug() << "ShellCorona screenForContainment: " << containment << " Last screen is " << containment->lastScreen();
+    for (int i = 0, count = m_screenConfiguration->outputs().count(); i<count; ++i) {
         if (containment->lastScreen() == i &&
             (containment->activity() == m_activityConsumer->currentActivity() ||
             containment->containmentType() == Plasma::Types::PanelContainment || containment->containmentType() == Plasma::Types::CustomPanelContainment)) {
             return i;
         }
-        ++i;
     }
 
     return -1;
